# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/kv/v1beta1/kv.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class Pairs(betterproto.Message):
    """Pairs defines a repeated slice of Pair objects."""

    pairs: List["Pair"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Pair(betterproto.Message):
    """Pair defines a key/value bytes tuple."""

    key: bytes = betterproto.bytes_field(1)
    value: bytes = betterproto.bytes_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/query/v1beta1/pagination.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class PageRequest(betterproto.Message):
    """
    PageRequest is to be embedded in gRPC request messages for efficient
    pagination. Ex:  message SomeRequest {          Foo some_parameter = 1;
    PageRequest pagination = 2;  }
    """

    key: bytes = betterproto.bytes_field(1)
    """
    key is a value returned in PageResponse.next_key to begin querying the next
    page most efficiently. Only one of offset or key should be set.
    """

    offset: int = betterproto.uint64_field(2)
    """
    offset is a numeric offset that can be used when key is unavailable. It is
    less efficient than using key. Only one of offset or key should be set.
    """

    limit: int = betterproto.uint64_field(3)
    """
    limit is the total number of results to be returned in the result page. If
    left empty it will default to a value to be set by each app.
    """

    count_total: bool = betterproto.bool_field(4)
    """
    count_total is set to true  to indicate that the result set should include
    a count of the total number of items available for pagination in UIs.
    count_total is only respected when offset is used. It is ignored when key
    is set.
    """

    reverse: bool = betterproto.bool_field(5)
    """
    reverse is set to true if results are to be returned in the descending
    order. Since: cosmos-sdk 0.43
    """


@dataclass(eq=False, repr=False)
class PageResponse(betterproto.Message):
    """
    PageResponse is to be embedded in gRPC response messages where the
    corresponding request message has used PageRequest.  message SomeResponse {
    repeated Bar results = 1;          PageResponse page = 2;  }
    """

    next_key: bytes = betterproto.bytes_field(1)
    """
    next_key is the key to be passed to PageRequest.key to query the next page
    most efficiently
    """

    total: int = betterproto.uint64_field(2)
    """
    total is total number of results available if PageRequest.count_total was
    set, its value is undefined otherwise
    """



from abc import abstractmethod

from betterproto import Message, Casing


class BaseMessageWrapper(Message):
    @property
    @abstractmethod
    def type_url(self):
        return ""

    @property
    @abstractmethod
    def legacy_url(self):
        return ""

    def to_data(self):
        return {"@type": self.type_url} | self.to_dict(include_default_values=True)

    def to_legacy_codec(self):
        return {"type": self.legacy_url, "value": self.to_dict(include_default_values=True, casing=Casing.SNAKE)}



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/store/v1beta1/commit_info.proto, cosmos/base/store/v1beta1/listening.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class CommitInfo(betterproto.Message):
    """
    CommitInfo defines commit information used by the multi-store when
    committing a version/height.
    """

    version: int = betterproto.int64_field(1)
    store_infos: List["StoreInfo"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class StoreInfo(betterproto.Message):
    """
    StoreInfo defines store-specific commit information. It contains a
    reference between a store name and the commit ID.
    """

    name: str = betterproto.string_field(1)
    commit_id: "CommitId" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CommitId(betterproto.Message):
    """
    CommitID defines the committment information when a specific store is
    committed.
    """

    version: int = betterproto.int64_field(1)
    hash: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class StoreKvPair(betterproto.Message):
    """
    StoreKVPair is a KVStore KVPair used for listening to state changes (Sets
    and Deletes) It optionally includes the StoreKey for the originating
    KVStore and a Boolean flag to distinguish between Sets and Deletes Since:
    cosmos-sdk 0.43
    """

    store_key: str = betterproto.string_field(1)
    delete: bool = betterproto.bool_field(2)
    key: bytes = betterproto.bytes_field(3)
    value: bytes = betterproto.bytes_field(4)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/evidence/v1beta1/evidence.proto, cosmos/evidence/v1beta1/genesis.proto, cosmos/evidence/v1beta1/query.proto, cosmos/evidence/v1beta1/tx.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base.query import v1beta1 as __base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class MsgSubmitEvidence(betterproto.Message):
    """
    MsgSubmitEvidence represents a message that supports submitting arbitrary
    Evidence of misbehavior such as equivocation or counterfactual signing.
    """

    submitter: str = betterproto.string_field(1)
    evidence: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class MsgSubmitEvidenceResponse(betterproto.Message):
    """
    MsgSubmitEvidenceResponse defines the Msg/SubmitEvidence response type.
    """

    hash: bytes = betterproto.bytes_field(4)
    """hash defines the hash of the evidence."""


@dataclass(eq=False, repr=False)
class Equivocation(betterproto.Message):
    """
    Equivocation implements the Evidence interface and defines evidence of
    double signing misbehavior.
    """

    height: int = betterproto.int64_field(1)
    time: datetime = betterproto.message_field(2)
    power: int = betterproto.int64_field(3)
    consensus_address: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class QueryEvidenceRequest(betterproto.Message):
    """
    QueryEvidenceRequest is the request type for the Query/Evidence RPC method.
    """

    evidence_hash: bytes = betterproto.bytes_field(1)
    """evidence_hash defines the hash of the requested evidence."""


@dataclass(eq=False, repr=False)
class QueryEvidenceResponse(betterproto.Message):
    """
    QueryEvidenceResponse is the response type for the Query/Evidence RPC
    method.
    """

    evidence: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    """evidence returns the requested evidence."""


@dataclass(eq=False, repr=False)
class QueryAllEvidenceRequest(betterproto.Message):
    """
    QueryEvidenceRequest is the request type for the Query/AllEvidence RPC
    method.
    """

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(1)
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryAllEvidenceResponse(betterproto.Message):
    """
    QueryAllEvidenceResponse is the response type for the Query/AllEvidence RPC
    method.
    """

    evidence: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(1)
    """evidence returns all evidences."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the evidence module's genesis state."""

    evidence: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(1)
    """evidence defines all the evidence at genesis."""


class MsgStub(betterproto.ServiceStub):
    async def submit_evidence(
        self,
        msg_submit_evidence: "MsgSubmitEvidence",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgSubmitEvidenceResponse":
        return await self._unary_unary(
            "/cosmos.evidence.v1beta1.Msg/SubmitEvidence",
            msg_submit_evidence,
            MsgSubmitEvidenceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def evidence(
        self,
        query_evidence_request: "QueryEvidenceRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryEvidenceResponse":
        return await self._unary_unary(
            "/cosmos.evidence.v1beta1.Query/Evidence",
            query_evidence_request,
            QueryEvidenceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def all_evidence(
        self,
        query_all_evidence_request: "QueryAllEvidenceRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAllEvidenceResponse":
        return await self._unary_unary(
            "/cosmos.evidence.v1beta1.Query/AllEvidence",
            query_all_evidence_request,
            QueryAllEvidenceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def submit_evidence(self, msg_submit_evidence: "MsgSubmitEvidence") -> "MsgSubmitEvidenceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_submit_evidence(
        self,
        stream: "grpclib.server.Stream[MsgSubmitEvidence, MsgSubmitEvidenceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.submit_evidence(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.evidence.v1beta1.Msg/SubmitEvidence": grpclib.const.Handler(
                self.__rpc_submit_evidence,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgSubmitEvidence,
                MsgSubmitEvidenceResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def evidence(self, query_evidence_request: "QueryEvidenceRequest") -> "QueryEvidenceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def all_evidence(self, query_all_evidence_request: "QueryAllEvidenceRequest") -> "QueryAllEvidenceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_evidence(
        self,
        stream: "grpclib.server.Stream[QueryEvidenceRequest, QueryEvidenceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.evidence(request)
        await stream.send_message(response)

    async def __rpc_all_evidence(
        self,
        stream: "grpclib.server.Stream[QueryAllEvidenceRequest, QueryAllEvidenceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.all_evidence(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.evidence.v1beta1.Query/Evidence": grpclib.const.Handler(
                self.__rpc_evidence,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryEvidenceRequest,
                QueryEvidenceResponse,
            ),
            "/cosmos.evidence.v1beta1.Query/AllEvidence": grpclib.const.Handler(
                self.__rpc_all_evidence,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAllEvidenceRequest,
                QueryAllEvidenceResponse,
            ),
        }



from bech32 import bech32_encode, bech32_decode, convertbits

from .constants import BECH32_ADDR_ACC_PREFIX, BECH32_ADDR_CONS_PREFIX, BECH32_ADDR_VAL_PREFIX
from ..exceptions import ConvertError, DecodeError


class Address:
    """Class for wrapping address. Contains methods to encode or decode Bech32.

    Attributes:
        addr:
    """

    def __init__(self, addr: bytes) -> None:
        self.addr = addr

    def __eq__(self, o: "Address") -> bool:
        return self.addr == o.addr

    @classmethod
    def _from_bech32(cls, bech: str, prefix: str) -> "Address":
        hrp, bz = bech32_decode(bech)
        assert hrp == prefix, "Invalid bech32 prefix"
        if bz is None:
            raise DecodeError("Cannot decode bech32")
        eight_bit_r = convertbits(bz, 5, 8, False)
        if eight_bit_r is None:
            raise ConvertError("Cannot convert to 8 bit")

        return cls(bytes(eight_bit_r))

    @classmethod
    def from_acc_bech32(cls, bech: str) -> "Address":
        """Creates an address instance from a bech32 account address.

        Args:
            bech: Bech32 account address.

        Returns:
            An Address instance.
        """

        return cls._from_bech32(bech, BECH32_ADDR_ACC_PREFIX)

    @classmethod
    def from_val_bech32(cls, bech: str) -> "Address":
        """Creates an address instance from a bech32 validator address.

        Args:
            bech: Bech32 validator address.

        Returns:
            An Address instance.
        """

        return cls._from_bech32(bech, BECH32_ADDR_VAL_PREFIX)

    @classmethod
    def from_cons_bech32(cls, bech: str) -> "Address":
        """Creates an address instance from a bech32 validator consensus address.

        Args:
            bech: Bech32 validator consensus address.

        Returns:
            An Address instance.
        """

        return cls._from_bech32(bech, BECH32_ADDR_CONS_PREFIX)

    def _to_bech32(self, prefix: str) -> str:
        five_bit_r = convertbits(self.addr, 8, 5)
        assert five_bit_r is not None, "Unsuccessful bech32.convertbits call"
        return bech32_encode(prefix, five_bit_r)

    def to_acc_bech32(self) -> str:
        """Returns a bech32 account address.

        Returns:
            A bech32 address as a string.
        """

        return self._to_bech32(BECH32_ADDR_ACC_PREFIX)

    def to_val_bech32(self) -> str:
        """Returns a bech32 validator address.

        Returns:
            A bech32 address as a string.
        """

        return self._to_bech32(BECH32_ADDR_VAL_PREFIX)

    def to_cons_bech32(self) -> str:
        """Returns a bech32 validator consensus address.

        Returns:
            A bech32 address as a string.
        """

        return self._to_bech32(BECH32_ADDR_CONS_PREFIX)

    def to_hex(self) -> str:
        """Returns a hex representation of the address.

        Returns:
            A hex representation of address.
        """

        return self.addr.hex()



from typing import Optional

LEDGER_RETURN_CODES = {
    "0x6400": "Execution error",
    "0x6700": "Wrong length",
    "0x6802": "Error deriving keys",
    "0x6982": "Empty buffer",
    "0x6983": "Output buffer too small",
    "0x6984": "Data is invalid",
    "0x6985": "Conditions not satisfied",
    "0x6986": "Command not allowed",
    "0x6a80": "Bad key handle",
    "0x6b00": "Invalid P1/P2",
    "0x6d00": "Instruction not supported",
    "0x6e00": "CLA not supported",
    "0x6f00": "Unknown",
    "0x6f01": "Sign/verify error",
    "0x9000": "Success",
    "0x9001": "Device is busy",
}


class PybandError(Exception):
    pass


class ValueTooLargeError(PybandError):
    pass


class EmptyMsgError(PybandError):
    pass


class NotFoundError(PybandError):
    pass


class UndefinedError(PybandError):
    pass


class DecodeError(PybandError):
    pass


class ConvertError(PybandError):
    pass


class SchemaError(PybandError):
    pass


class NotBip44Error(PybandError):
    pass


class IncorrectLengthError(PybandError):
    pass


class CosmosAppError(PybandError):
    def __init__(self, return_code: Optional[str] = None):
        if return_code:
            self.msg = LEDGER_RETURN_CODES.get(return_code, "Unknown error")
        else:
            self.msg = "Unknown error"

    def __str__(self):
        return self.msg



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: tendermint/p2p/types.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class NetAddress(betterproto.Message):
    id: str = betterproto.string_field(1)
    ip: str = betterproto.string_field(2)
    port: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class ProtocolVersion(betterproto.Message):
    p2_p: int = betterproto.uint64_field(1)
    block: int = betterproto.uint64_field(2)
    app: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class DefaultNodeInfo(betterproto.Message):
    protocol_version: "ProtocolVersion" = betterproto.message_field(1)
    default_node_id: str = betterproto.string_field(2)
    listen_addr: str = betterproto.string_field(3)
    network: str = betterproto.string_field(4)
    version: str = betterproto.string_field(5)
    channels: bytes = betterproto.bytes_field(6)
    moniker: str = betterproto.string_field(7)
    other: "DefaultNodeInfoOther" = betterproto.message_field(8)


@dataclass(eq=False, repr=False)
class DefaultNodeInfoOther(betterproto.Message):
    tx_index: str = betterproto.string_field(1)
    rpc_address: str = betterproto.string_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/tx/v1beta1/service.proto, cosmos/tx/v1beta1/tx.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....tendermint import types as ___tendermint_types__
from ...base import v1beta1 as __base_v1_beta1__
from ...base.abci import v1beta1 as __base_abci_v1_beta1__
from ...base.query import v1beta1 as __base_query_v1_beta1__
from ...crypto.multisig import v1beta1 as __crypto_multisig_v1_beta1__
from ..signing import v1beta1 as _signing_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class OrderBy(betterproto.Enum):
    """OrderBy defines the sorting order"""

    ORDER_BY_UNSPECIFIED = 0
    """
    ORDER_BY_UNSPECIFIED specifies an unknown sorting order. OrderBy defaults
    to ASC in this case.
    """

    ORDER_BY_ASC = 1
    """ORDER_BY_ASC defines ascending order"""

    ORDER_BY_DESC = 2
    """ORDER_BY_DESC defines descending order"""


class BroadcastMode(betterproto.Enum):
    """
    BroadcastMode specifies the broadcast mode for the TxService.Broadcast RPC
    method.
    """

    BROADCAST_MODE_UNSPECIFIED = 0
    """zero-value for mode ordering"""

    BROADCAST_MODE_BLOCK = 1
    """
    BROADCAST_MODE_BLOCK defines a tx broadcasting mode where the client waits
    for the tx to be committed in a block.
    """

    BROADCAST_MODE_SYNC = 2
    """
    BROADCAST_MODE_SYNC defines a tx broadcasting mode where the client waits
    for a CheckTx execution response only.
    """

    BROADCAST_MODE_ASYNC = 3
    """
    BROADCAST_MODE_ASYNC defines a tx broadcasting mode where the client
    returns immediately.
    """


@dataclass(eq=False, repr=False)
class Tx(betterproto.Message):
    """Tx is the standard type used for broadcasting transactions."""

    body: "TxBody" = betterproto.message_field(1)
    """body is the processable content of the transaction"""

    auth_info: "AuthInfo" = betterproto.message_field(2)
    """
    auth_info is the authorization related content of the transaction,
    specifically signers, signer modes and fee
    """

    signatures: List[bytes] = betterproto.bytes_field(3)
    """
    signatures is a list of signatures that matches the length and order of
    AuthInfo's signer_infos to allow connecting signature meta information like
    public key and signing mode by position.
    """


@dataclass(eq=False, repr=False)
class TxRaw(betterproto.Message):
    """
    TxRaw is a variant of Tx that pins the signer's exact binary representation
    of body and auth_info. This is used for signing, broadcasting and
    verification. The binary `serialize(tx: TxRaw)` is stored in Tendermint and
    the hash `sha256(serialize(tx: TxRaw))` becomes the "txhash", commonly used
    as the transaction ID.
    """

    body_bytes: bytes = betterproto.bytes_field(1)
    """
    body_bytes is a protobuf serialization of a TxBody that matches the
    representation in SignDoc.
    """

    auth_info_bytes: bytes = betterproto.bytes_field(2)
    """
    auth_info_bytes is a protobuf serialization of an AuthInfo that matches the
    representation in SignDoc.
    """

    signatures: List[bytes] = betterproto.bytes_field(3)
    """
    signatures is a list of signatures that matches the length and order of
    AuthInfo's signer_infos to allow connecting signature meta information like
    public key and signing mode by position.
    """


@dataclass(eq=False, repr=False)
class SignDoc(betterproto.Message):
    """
    SignDoc is the type used for generating sign bytes for SIGN_MODE_DIRECT.
    """

    body_bytes: bytes = betterproto.bytes_field(1)
    """
    body_bytes is protobuf serialization of a TxBody that matches the
    representation in TxRaw.
    """

    auth_info_bytes: bytes = betterproto.bytes_field(2)
    """
    auth_info_bytes is a protobuf serialization of an AuthInfo that matches the
    representation in TxRaw.
    """

    chain_id: str = betterproto.string_field(3)
    """
    chain_id is the unique identifier of the chain this transaction targets. It
    prevents signed transactions from being used on another chain by an
    attacker
    """

    account_number: int = betterproto.uint64_field(4)
    """account_number is the account number of the account in state"""


@dataclass(eq=False, repr=False)
class TxBody(betterproto.Message):
    """TxBody is the body of a transaction that all signers sign over."""

    messages: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(1)
    """
    messages is a list of messages to be executed. The required signers of
    those messages define the number and order of elements in AuthInfo's
    signer_infos and Tx's signatures. Each required signer address is added to
    the list only the first time it occurs. By convention, the first required
    signer (usually from the first message) is referred to as the primary
    signer and pays the fee for the whole transaction.
    """

    memo: str = betterproto.string_field(2)
    """
    memo is any arbitrary note/comment to be added to the transaction. WARNING:
    in clients, any publicly exposed text should not be called memo, but should
    be called `note` instead (see https://github.com/cosmos/cosmos-
    sdk/issues/9122).
    """

    timeout_height: int = betterproto.uint64_field(3)
    """
    timeout is the block height after which this transaction will not be
    processed by the chain
    """

    extension_options: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(1023)
    """
    extension_options are arbitrary options that can be added by chains when
    the default options are not sufficient. If any of these are present and
    can't be handled, the transaction will be rejected
    """

    non_critical_extension_options: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(2047)
    """
    extension_options are arbitrary options that can be added by chains when
    the default options are not sufficient. If any of these are present and
    can't be handled, they will be ignored
    """


@dataclass(eq=False, repr=False)
class AuthInfo(betterproto.Message):
    """
    AuthInfo describes the fee and signer modes that are used to sign a
    transaction.
    """

    signer_infos: List["SignerInfo"] = betterproto.message_field(1)
    """
    signer_infos defines the signing modes for the required signers. The number
    and order of elements must match the required signers from TxBody's
    messages. The first element is the primary signer and the one which pays
    the fee.
    """

    fee: "Fee" = betterproto.message_field(2)
    """
    Fee is the fee and gas limit for the transaction. The first signer is the
    primary signer and the one which pays the fee. The fee can be calculated
    based on the cost of evaluating the body and doing signature verification
    of the signers. This can be estimated via simulation.
    """


@dataclass(eq=False, repr=False)
class SignerInfo(betterproto.Message):
    """
    SignerInfo describes the public key and signing mode of a single top-level
    signer.
    """

    public_key: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    """
    public_key is the public key of the signer. It is optional for accounts
    that already exist in state. If unset, the verifier can use the required \
    signer address for this position and lookup the public key.
    """

    mode_info: "ModeInfo" = betterproto.message_field(2)
    """
    mode_info describes the signing mode of the signer and is a nested
    structure to support nested multisig pubkey's
    """

    sequence: int = betterproto.uint64_field(3)
    """
    sequence is the sequence of the account, which describes the number of
    committed transactions signed by a given address. It is used to prevent
    replay attacks.
    """


@dataclass(eq=False, repr=False)
class ModeInfo(betterproto.Message):
    """
    ModeInfo describes the signing mode of a single or nested multisig signer.
    """

    single: "ModeInfoSingle" = betterproto.message_field(1, group="sum")
    """single represents a single signer"""

    multi: "ModeInfoMulti" = betterproto.message_field(2, group="sum")
    """multi represents a nested multisig signer"""


@dataclass(eq=False, repr=False)
class ModeInfoSingle(betterproto.Message):
    """
    Single is the mode info for a single signer. It is structured as a message
    to allow for additional fields such as locale for SIGN_MODE_TEXTUAL in the
    future
    """

    mode: "_signing_v1_beta1__.SignMode" = betterproto.enum_field(1)
    """mode is the signing mode of the single signer"""


@dataclass(eq=False, repr=False)
class ModeInfoMulti(betterproto.Message):
    """Multi is the mode info for a multisig public key"""

    bitarray: "__crypto_multisig_v1_beta1__.CompactBitArray" = betterproto.message_field(1)
    """bitarray specifies which keys within the multisig are signing"""

    mode_infos: List["ModeInfo"] = betterproto.message_field(2)
    """
    mode_infos is the corresponding modes of the signers of the multisig which
    could include nested multisig public keys
    """


@dataclass(eq=False, repr=False)
class Fee(betterproto.Message):
    """
    Fee includes the amount of coins paid in fees and the maximum gas to be
    used by the transaction. The ratio yields an effective "gasprice", which
    must be above some miminum to be accepted into the mempool.
    """

    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(1)
    """amount is the amount of coins to be paid as a fee"""

    gas_limit: int = betterproto.uint64_field(2)
    """
    gas_limit is the maximum gas that can be used in transaction processing
    before an out of gas error occurs
    """

    payer: str = betterproto.string_field(3)
    """
    if unset, the first signer is responsible for paying the fees. If set, the
    specified account must pay the fees. the payer must be a tx signer (and
    thus have signed this field in AuthInfo). setting this field does *not*
    change the ordering of required signers for the transaction.
    """

    granter: str = betterproto.string_field(4)
    """
    if set, the fee payer (either the first signer or the value of the payer
    field) requests that a fee grant be used to pay fees instead of the fee
    payer's own balance. If an appropriate fee grant does not exist or the
    chain does not support fee grants, this will fail
    """


@dataclass(eq=False, repr=False)
class GetTxsEventRequest(betterproto.Message):
    """
    GetTxsEventRequest is the request type for the Service.TxsByEvents RPC
    method.
    """

    events: List[str] = betterproto.string_field(1)
    """events is the list of transaction event type."""

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines a pagination for the request."""

    order_by: "OrderBy" = betterproto.enum_field(3)


@dataclass(eq=False, repr=False)
class GetTxsEventResponse(betterproto.Message):
    """
    GetTxsEventResponse is the response type for the Service.TxsByEvents RPC
    method.
    """

    txs: List["Tx"] = betterproto.message_field(1)
    """txs is the list of queried transactions."""

    tx_responses: List["__base_abci_v1_beta1__.TxResponse"] = betterproto.message_field(2)
    """tx_responses is the list of queried TxResponses."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(3)
    """pagination defines a pagination for the response."""


@dataclass(eq=False, repr=False)
class BroadcastTxRequest(betterproto.Message):
    """
    BroadcastTxRequest is the request type for the Service.BroadcastTxRequest
    RPC method.
    """

    tx_bytes: bytes = betterproto.bytes_field(1)
    """tx_bytes is the raw transaction."""

    mode: "BroadcastMode" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class BroadcastTxResponse(betterproto.Message):
    """
    BroadcastTxResponse is the response type for the Service.BroadcastTx
    method.
    """

    tx_response: "__base_abci_v1_beta1__.TxResponse" = betterproto.message_field(1)
    """tx_response is the queried TxResponses."""


@dataclass(eq=False, repr=False)
class SimulateRequest(betterproto.Message):
    """
    SimulateRequest is the request type for the Service.Simulate RPC method.
    """

    tx: "Tx" = betterproto.message_field(1)
    """
    tx is the transaction to simulate. Deprecated. Send raw tx bytes instead.
    """

    tx_bytes: bytes = betterproto.bytes_field(2)
    """tx_bytes is the raw transaction. Since: cosmos-sdk 0.43"""

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("tx"):
            warnings.warn("SimulateRequest.tx is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class SimulateResponse(betterproto.Message):
    """
    SimulateResponse is the response type for the Service.SimulateRPC method.
    """

    gas_info: "__base_abci_v1_beta1__.GasInfo" = betterproto.message_field(1)
    """gas_info is the information about gas used in the simulation."""

    result: "__base_abci_v1_beta1__.Result" = betterproto.message_field(2)
    """result is the result of the simulation."""


@dataclass(eq=False, repr=False)
class GetTxRequest(betterproto.Message):
    """GetTxRequest is the request type for the Service.GetTx RPC method."""

    hash: str = betterproto.string_field(1)
    """hash is the tx hash to query, encoded as a hex string."""


@dataclass(eq=False, repr=False)
class GetTxResponse(betterproto.Message):
    """GetTxResponse is the response type for the Service.GetTx method."""

    tx: "Tx" = betterproto.message_field(1)
    """tx is the queried transaction."""

    tx_response: "__base_abci_v1_beta1__.TxResponse" = betterproto.message_field(2)
    """tx_response is the queried TxResponses."""


@dataclass(eq=False, repr=False)
class GetBlockWithTxsRequest(betterproto.Message):
    """
    GetBlockWithTxsRequest is the request type for the Service.GetBlockWithTxs
    RPC method. Since: cosmos-sdk 0.45.2
    """

    height: int = betterproto.int64_field(1)
    """height is the height of the block to query."""

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines a pagination for the request."""


@dataclass(eq=False, repr=False)
class GetBlockWithTxsResponse(betterproto.Message):
    """
    GetBlockWithTxsResponse is the response type for the
    Service.GetBlockWithTxs method. Since: cosmos-sdk 0.45.2
    """

    txs: List["Tx"] = betterproto.message_field(1)
    """txs are the transactions in the block."""

    block_id: "___tendermint_types__.BlockId" = betterproto.message_field(2)
    block: "___tendermint_types__.Block" = betterproto.message_field(3)
    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(4)
    """pagination defines a pagination for the response."""


class ServiceStub(betterproto.ServiceStub):
    async def simulate(
        self,
        simulate_request: "SimulateRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SimulateResponse":
        return await self._unary_unary(
            "/cosmos.tx.v1beta1.Service/Simulate",
            simulate_request,
            SimulateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_tx(
        self,
        get_tx_request: "GetTxRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetTxResponse":
        return await self._unary_unary(
            "/cosmos.tx.v1beta1.Service/GetTx",
            get_tx_request,
            GetTxResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def broadcast_tx(
        self,
        broadcast_tx_request: "BroadcastTxRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "BroadcastTxResponse":
        return await self._unary_unary(
            "/cosmos.tx.v1beta1.Service/BroadcastTx",
            broadcast_tx_request,
            BroadcastTxResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_txs_event(
        self,
        get_txs_event_request: "GetTxsEventRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetTxsEventResponse":
        return await self._unary_unary(
            "/cosmos.tx.v1beta1.Service/GetTxsEvent",
            get_txs_event_request,
            GetTxsEventResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_block_with_txs(
        self,
        get_block_with_txs_request: "GetBlockWithTxsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetBlockWithTxsResponse":
        return await self._unary_unary(
            "/cosmos.tx.v1beta1.Service/GetBlockWithTxs",
            get_block_with_txs_request,
            GetBlockWithTxsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class ServiceBase(ServiceBase):
    async def simulate(self, simulate_request: "SimulateRequest") -> "SimulateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_tx(self, get_tx_request: "GetTxRequest") -> "GetTxResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def broadcast_tx(self, broadcast_tx_request: "BroadcastTxRequest") -> "BroadcastTxResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_txs_event(self, get_txs_event_request: "GetTxsEventRequest") -> "GetTxsEventResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_block_with_txs(
        self, get_block_with_txs_request: "GetBlockWithTxsRequest"
    ) -> "GetBlockWithTxsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_simulate(self, stream: "grpclib.server.Stream[SimulateRequest, SimulateResponse]") -> None:
        request = await stream.recv_message()
        response = await self.simulate(request)
        await stream.send_message(response)

    async def __rpc_get_tx(self, stream: "grpclib.server.Stream[GetTxRequest, GetTxResponse]") -> None:
        request = await stream.recv_message()
        response = await self.get_tx(request)
        await stream.send_message(response)

    async def __rpc_broadcast_tx(
        self, stream: "grpclib.server.Stream[BroadcastTxRequest, BroadcastTxResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.broadcast_tx(request)
        await stream.send_message(response)

    async def __rpc_get_txs_event(
        self, stream: "grpclib.server.Stream[GetTxsEventRequest, GetTxsEventResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_txs_event(request)
        await stream.send_message(response)

    async def __rpc_get_block_with_txs(
        self,
        stream: "grpclib.server.Stream[GetBlockWithTxsRequest, GetBlockWithTxsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_block_with_txs(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.tx.v1beta1.Service/Simulate": grpclib.const.Handler(
                self.__rpc_simulate,
                grpclib.const.Cardinality.UNARY_UNARY,
                SimulateRequest,
                SimulateResponse,
            ),
            "/cosmos.tx.v1beta1.Service/GetTx": grpclib.const.Handler(
                self.__rpc_get_tx,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetTxRequest,
                GetTxResponse,
            ),
            "/cosmos.tx.v1beta1.Service/BroadcastTx": grpclib.const.Handler(
                self.__rpc_broadcast_tx,
                grpclib.const.Cardinality.UNARY_UNARY,
                BroadcastTxRequest,
                BroadcastTxResponse,
            ),
            "/cosmos.tx.v1beta1.Service/GetTxsEvent": grpclib.const.Handler(
                self.__rpc_get_txs_event,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetTxsEventRequest,
                GetTxsEventResponse,
            ),
            "/cosmos.tx.v1beta1.Service/GetBlockWithTxs": grpclib.const.Handler(
                self.__rpc_get_block_with_txs,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetBlockWithTxsRequest,
                GetBlockWithTxsResponse,
            ),
        }



from dataclasses import dataclass

from ....base import BaseMessageWrapper
from .....proto.cosmos.base import v1beta1 as __base_v1_beta1__
from .....proto.cosmos.base.query import v1beta1 as __base_query_v1_beta1__
from .....proto.cosmos.distribution.v1beta1 import MsgWithdrawDelegatorReward as MsgWithdrawDelegatorRewardProto

assert __base_v1_beta1__
assert __base_query_v1_beta1__


class MsgWithdrawDelegationReward(BaseMessageWrapper, MsgWithdrawDelegatorRewardProto):
    @property
    def type_url(self):
        return "/cosmos.distribution.v1beta1.MsgWithdrawDelegatorReward"

    @property
    def legacy_url(self):
        return "cosmos-sdk/MsgWithdrawDelegationReward"



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/auth/v1beta1/auth.proto, cosmos/auth/v1beta1/genesis.proto, cosmos/auth/v1beta1/query.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base.query import v1beta1 as __base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class BaseAccount(betterproto.Message):
    """
    BaseAccount defines a base account type. It contains all the necessary
    fields for basic account functionality. Any custom account type should
    extend this type for additional functionality (e.g. vesting).
    """

    address: str = betterproto.string_field(1)
    pub_key: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)
    account_number: int = betterproto.uint64_field(3)
    sequence: int = betterproto.uint64_field(4)


@dataclass(eq=False, repr=False)
class ModuleAccount(betterproto.Message):
    """
    ModuleAccount defines an account for modules that holds coins on a pool.
    """

    base_account: "BaseAccount" = betterproto.message_field(1)
    name: str = betterproto.string_field(2)
    permissions: List[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the parameters for the auth module."""

    max_memo_characters: int = betterproto.uint64_field(1)
    tx_sig_limit: int = betterproto.uint64_field(2)
    tx_size_cost_per_byte: int = betterproto.uint64_field(3)
    sig_verify_cost_ed25519: int = betterproto.uint64_field(4)
    sig_verify_cost_secp256_k1: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class QueryAccountsRequest(betterproto.Message):
    """
    QueryAccountsRequest is the request type for the Query/Accounts RPC method.
    Since: cosmos-sdk 0.43
    """

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(1)
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryAccountsResponse(betterproto.Message):
    """
    QueryAccountsResponse is the response type for the Query/Accounts RPC
    method. Since: cosmos-sdk 0.43
    """

    accounts: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(1)
    """accounts are the existing accounts"""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryAccountRequest(betterproto.Message):
    """
    QueryAccountRequest is the request type for the Query/Account RPC method.
    """

    address: str = betterproto.string_field(1)
    """address defines the address to query for."""


@dataclass(eq=False, repr=False)
class QueryAccountResponse(betterproto.Message):
    """
    QueryAccountResponse is the response type for the Query/Account RPC method.
    """

    account: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    """account defines the account of the corresponding address."""


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method.
    """

    params: "Params" = betterproto.message_field(1)
    """params defines the parameters of the module."""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the auth module's genesis state."""

    params: "Params" = betterproto.message_field(1)
    """params defines all the paramaters of the module."""

    accounts: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(2)
    """accounts are the accounts present at genesis."""


class QueryStub(betterproto.ServiceStub):
    async def accounts(
        self,
        query_accounts_request: "QueryAccountsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAccountsResponse":
        return await self._unary_unary(
            "/cosmos.auth.v1beta1.Query/Accounts",
            query_accounts_request,
            QueryAccountsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def account(
        self,
        query_account_request: "QueryAccountRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAccountResponse":
        return await self._unary_unary(
            "/cosmos.auth.v1beta1.Query/Account",
            query_account_request,
            QueryAccountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/cosmos.auth.v1beta1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def accounts(self, query_accounts_request: "QueryAccountsRequest") -> "QueryAccountsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def account(self, query_account_request: "QueryAccountRequest") -> "QueryAccountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(self, query_params_request: "QueryParamsRequest") -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_accounts(
        self,
        stream: "grpclib.server.Stream[QueryAccountsRequest, QueryAccountsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.accounts(request)
        await stream.send_message(response)

    async def __rpc_account(self, stream: "grpclib.server.Stream[QueryAccountRequest, QueryAccountResponse]") -> None:
        request = await stream.recv_message()
        response = await self.account(request)
        await stream.send_message(response)

    async def __rpc_params(self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.auth.v1beta1.Query/Accounts": grpclib.const.Handler(
                self.__rpc_accounts,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAccountsRequest,
                QueryAccountsResponse,
            ),
            "/cosmos.auth.v1beta1.Query/Account": grpclib.const.Handler(
                self.__rpc_account,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAccountRequest,
                QueryAccountResponse,
            ),
            "/cosmos.auth.v1beta1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/genutil/v1beta1/genesis.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the raw genesis transaction in JSON."""

    gen_txs: List[bytes] = betterproto.bytes_field(1)
    """gen_txs defines the genesis transactions."""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/tendermint/v1beta1/query.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from .....tendermint import (
    p2p as ____tendermint_p2_p__,
    types as ____tendermint_types__,
)
from ...query import v1beta1 as __query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class GetValidatorSetByHeightRequest(betterproto.Message):
    """
    GetValidatorSetByHeightRequest is the request type for the
    Query/GetValidatorSetByHeight RPC method.
    """

    height: int = betterproto.int64_field(1)
    pagination: "__query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class GetValidatorSetByHeightResponse(betterproto.Message):
    """
    GetValidatorSetByHeightResponse is the response type for the
    Query/GetValidatorSetByHeight RPC method.
    """

    block_height: int = betterproto.int64_field(1)
    validators: List["Validator"] = betterproto.message_field(2)
    pagination: "__query_v1_beta1__.PageResponse" = betterproto.message_field(3)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class GetLatestValidatorSetRequest(betterproto.Message):
    """
    GetLatestValidatorSetRequest is the request type for the
    Query/GetValidatorSetByHeight RPC method.
    """

    pagination: "__query_v1_beta1__.PageRequest" = betterproto.message_field(1)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class GetLatestValidatorSetResponse(betterproto.Message):
    """
    GetLatestValidatorSetResponse is the response type for the
    Query/GetValidatorSetByHeight RPC method.
    """

    block_height: int = betterproto.int64_field(1)
    validators: List["Validator"] = betterproto.message_field(2)
    pagination: "__query_v1_beta1__.PageResponse" = betterproto.message_field(3)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class Validator(betterproto.Message):
    """Validator is the type for the validator-set."""

    address: str = betterproto.string_field(1)
    pub_key: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)
    voting_power: int = betterproto.int64_field(3)
    proposer_priority: int = betterproto.int64_field(4)


@dataclass(eq=False, repr=False)
class GetBlockByHeightRequest(betterproto.Message):
    """
    GetBlockByHeightRequest is the request type for the Query/GetBlockByHeight
    RPC method.
    """

    height: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class GetBlockByHeightResponse(betterproto.Message):
    """
    GetBlockByHeightResponse is the response type for the
    Query/GetBlockByHeight RPC method.
    """

    block_id: "____tendermint_types__.BlockId" = betterproto.message_field(1)
    block: "____tendermint_types__.Block" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GetLatestBlockRequest(betterproto.Message):
    """
    GetLatestBlockRequest is the request type for the Query/GetLatestBlock RPC
    method.
    """

    pass


@dataclass(eq=False, repr=False)
class GetLatestBlockResponse(betterproto.Message):
    """
    GetLatestBlockResponse is the response type for the Query/GetLatestBlock
    RPC method.
    """

    block_id: "____tendermint_types__.BlockId" = betterproto.message_field(1)
    block: "____tendermint_types__.Block" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GetSyncingRequest(betterproto.Message):
    """
    GetSyncingRequest is the request type for the Query/GetSyncing RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class GetSyncingResponse(betterproto.Message):
    """
    GetSyncingResponse is the response type for the Query/GetSyncing RPC
    method.
    """

    syncing: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class GetNodeInfoRequest(betterproto.Message):
    """
    GetNodeInfoRequest is the request type for the Query/GetNodeInfo RPC
    method.
    """

    pass


@dataclass(eq=False, repr=False)
class GetNodeInfoResponse(betterproto.Message):
    """
    GetNodeInfoResponse is the request type for the Query/GetNodeInfo RPC
    method.
    """

    default_node_info: "____tendermint_p2_p__.DefaultNodeInfo" = betterproto.message_field(1)
    application_version: "VersionInfo" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class VersionInfo(betterproto.Message):
    """VersionInfo is the type for the GetNodeInfoResponse message."""

    name: str = betterproto.string_field(1)
    app_name: str = betterproto.string_field(2)
    version: str = betterproto.string_field(3)
    git_commit: str = betterproto.string_field(4)
    build_tags: str = betterproto.string_field(5)
    go_version: str = betterproto.string_field(6)
    build_deps: List["Module"] = betterproto.message_field(7)
    cosmos_sdk_version: str = betterproto.string_field(8)
    """Since: cosmos-sdk 0.43"""


@dataclass(eq=False, repr=False)
class Module(betterproto.Message):
    """Module is the type for VersionInfo"""

    path: str = betterproto.string_field(1)
    """module path"""

    version: str = betterproto.string_field(2)
    """module version"""

    sum: str = betterproto.string_field(3)
    """checksum"""


class ServiceStub(betterproto.ServiceStub):
    async def get_node_info(
        self,
        get_node_info_request: "GetNodeInfoRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetNodeInfoResponse":
        return await self._unary_unary(
            "/cosmos.base.tendermint.v1beta1.Service/GetNodeInfo",
            get_node_info_request,
            GetNodeInfoResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_syncing(
        self,
        get_syncing_request: "GetSyncingRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetSyncingResponse":
        return await self._unary_unary(
            "/cosmos.base.tendermint.v1beta1.Service/GetSyncing",
            get_syncing_request,
            GetSyncingResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_latest_block(
        self,
        get_latest_block_request: "GetLatestBlockRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetLatestBlockResponse":
        return await self._unary_unary(
            "/cosmos.base.tendermint.v1beta1.Service/GetLatestBlock",
            get_latest_block_request,
            GetLatestBlockResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_block_by_height(
        self,
        get_block_by_height_request: "GetBlockByHeightRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetBlockByHeightResponse":
        return await self._unary_unary(
            "/cosmos.base.tendermint.v1beta1.Service/GetBlockByHeight",
            get_block_by_height_request,
            GetBlockByHeightResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_latest_validator_set(
        self,
        get_latest_validator_set_request: "GetLatestValidatorSetRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetLatestValidatorSetResponse":
        return await self._unary_unary(
            "/cosmos.base.tendermint.v1beta1.Service/GetLatestValidatorSet",
            get_latest_validator_set_request,
            GetLatestValidatorSetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_validator_set_by_height(
        self,
        get_validator_set_by_height_request: "GetValidatorSetByHeightRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetValidatorSetByHeightResponse":
        return await self._unary_unary(
            "/cosmos.base.tendermint.v1beta1.Service/GetValidatorSetByHeight",
            get_validator_set_by_height_request,
            GetValidatorSetByHeightResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class ServiceBase(ServiceBase):
    async def get_node_info(self, get_node_info_request: "GetNodeInfoRequest") -> "GetNodeInfoResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_syncing(self, get_syncing_request: "GetSyncingRequest") -> "GetSyncingResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_latest_block(self, get_latest_block_request: "GetLatestBlockRequest") -> "GetLatestBlockResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_block_by_height(
        self, get_block_by_height_request: "GetBlockByHeightRequest"
    ) -> "GetBlockByHeightResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_latest_validator_set(
        self, get_latest_validator_set_request: "GetLatestValidatorSetRequest"
    ) -> "GetLatestValidatorSetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_validator_set_by_height(
        self, get_validator_set_by_height_request: "GetValidatorSetByHeightRequest"
    ) -> "GetValidatorSetByHeightResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_node_info(
        self, stream: "grpclib.server.Stream[GetNodeInfoRequest, GetNodeInfoResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_node_info(request)
        await stream.send_message(response)

    async def __rpc_get_syncing(self, stream: "grpclib.server.Stream[GetSyncingRequest, GetSyncingResponse]") -> None:
        request = await stream.recv_message()
        response = await self.get_syncing(request)
        await stream.send_message(response)

    async def __rpc_get_latest_block(
        self,
        stream: "grpclib.server.Stream[GetLatestBlockRequest, GetLatestBlockResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_latest_block(request)
        await stream.send_message(response)

    async def __rpc_get_block_by_height(
        self,
        stream: "grpclib.server.Stream[GetBlockByHeightRequest, GetBlockByHeightResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_block_by_height(request)
        await stream.send_message(response)

    async def __rpc_get_latest_validator_set(
        self,
        stream: "grpclib.server.Stream[GetLatestValidatorSetRequest, GetLatestValidatorSetResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_latest_validator_set(request)
        await stream.send_message(response)

    async def __rpc_get_validator_set_by_height(
        self,
        stream: "grpclib.server.Stream[GetValidatorSetByHeightRequest, GetValidatorSetByHeightResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_validator_set_by_height(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.base.tendermint.v1beta1.Service/GetNodeInfo": grpclib.const.Handler(
                self.__rpc_get_node_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetNodeInfoRequest,
                GetNodeInfoResponse,
            ),
            "/cosmos.base.tendermint.v1beta1.Service/GetSyncing": grpclib.const.Handler(
                self.__rpc_get_syncing,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetSyncingRequest,
                GetSyncingResponse,
            ),
            "/cosmos.base.tendermint.v1beta1.Service/GetLatestBlock": grpclib.const.Handler(
                self.__rpc_get_latest_block,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetLatestBlockRequest,
                GetLatestBlockResponse,
            ),
            "/cosmos.base.tendermint.v1beta1.Service/GetBlockByHeight": grpclib.const.Handler(
                self.__rpc_get_block_by_height,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetBlockByHeightRequest,
                GetBlockByHeightResponse,
            ),
            "/cosmos.base.tendermint.v1beta1.Service/GetLatestValidatorSet": grpclib.const.Handler(
                self.__rpc_get_latest_validator_set,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetLatestValidatorSetRequest,
                GetLatestValidatorSetResponse,
            ),
            "/cosmos.base.tendermint.v1beta1.Service/GetValidatorSetByHeight": grpclib.const.Handler(
                self.__rpc_get_validator_set_by_height,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetValidatorSetByHeightRequest,
                GetValidatorSetByHeightResponse,
            ),
        }



import hashlib
from typing import Tuple, Optional

from bip32 import BIP32
from ecdsa.curves import SECP256k1
from ecdsa.keys import SigningKey
from ecdsa.util import sigencode_string_canonize
from mnemonic import Mnemonic

from .constants import DEFAULT_DERIVATION_PATH
from .public_key import PublicKey


class PrivateKey:
    """Class for wrapping SigningKey, used to sign messages and derive the associated public key.

    Attributes:
        signing_key: The ecdsa SigningKey instance.
    """

    def __init__(self, _error_do_not_use_init_directly=None) -> None:
        """Unsupported, please use from_mnemonic to initialize."""

        if not _error_do_not_use_init_directly:
            raise TypeError("Please use PrivateKey.from_mnemonic() to construct me")
        self.signing_key: Optional[SigningKey] = None

    @classmethod
    def generate(cls, path=DEFAULT_DERIVATION_PATH) -> Tuple[str, "PrivateKey"]:
        """Generates a new private key with a derivation path and random mnemonic phrase.

        Args:
            path: The derivation path, if omitted, defaults to Band's default HD prefix 494 with all other indexes being zeroes.

        Returns:
            A tuple containing the mnemonic phrase and the PrivateKey instance.
        """

        phrase = Mnemonic(language="english").generate(strength=256)
        return phrase, cls.from_mnemonic(phrase, path)

    @classmethod
    def from_mnemonic(cls, words: str, path=DEFAULT_DERIVATION_PATH) -> "PrivateKey":
        """Create a PrivateKey instance from a given mnemonic phrase and derivation path.

        Args:
            words: The mnemonic phrase.
            path: The derivation path, if omitted, defaults to Band's default HD prefix 494 with all other indexes being zeroes.

        Returns:
            A PrivateKey instance.
        """

        seed = Mnemonic("english").to_seed(words)
        self = cls(_error_do_not_use_init_directly=True)
        self.signing_key = SigningKey.from_string(
            BIP32.from_seed(seed).get_privkey_from_path(path), curve=SECP256k1, hashfunc=hashlib.sha256
        )
        return self

    @classmethod
    def from_hex(cls, priv: str) -> "PrivateKey":
        """Create a PrivateKey instance from a hex representation of the signing key.

        Args:
            priv: A hex representation of the signing key.

        Returns:
            A PrivateKey instance.
        """

        self = cls(_error_do_not_use_init_directly=True)
        self.signing_key = SigningKey.from_string(bytes.fromhex(priv), curve=SECP256k1, hashfunc=hashlib.sha256)
        return self

    def to_hex(self) -> str:
        """Returns a hex representation of the signing key.

        Returns:
            A hex representation of signing key.
        """

        return self.signing_key.to_string().hex()

    def to_public_key(self) -> PublicKey:
        """Gets the associated PublicKey.

        Returns:
            A PublicKey instance associated with the object's PrivateKey.
        """

        public_key = PublicKey(_error_do_not_use_init_directly=True)
        public_key.verify_key = self.signing_key.get_verifying_key()
        return public_key

    def sign(self, msg: bytes) -> bytes:
        """Sign the given message using the edcsa sign_deterministic function.

        Args:
            msg: Message to be signed and hashed.

        Returns:
            A signature of this private key over the given message.
        """

        return self.signing_key.sign_deterministic(msg, hashfunc=hashlib.sha256, sigencode=sigencode_string_canonize)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/gov/v1beta1/genesis.proto, cosmos/gov/v1beta1/gov.proto, cosmos/gov/v1beta1/query.proto, cosmos/gov/v1beta1/tx.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base import v1beta1 as __base_v1_beta1__
from ...base.query import v1beta1 as __base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class VoteOption(betterproto.Enum):
    """
    VoteOption enumerates the valid vote options for a given governance
    proposal.
    """

    VOTE_OPTION_UNSPECIFIED = 0
    """VOTE_OPTION_UNSPECIFIED defines a no-op vote option."""

    VOTE_OPTION_YES = 1
    """VOTE_OPTION_YES defines a yes vote option."""

    VOTE_OPTION_ABSTAIN = 2
    """VOTE_OPTION_ABSTAIN defines an abstain vote option."""

    VOTE_OPTION_NO = 3
    """VOTE_OPTION_NO defines a no vote option."""

    VOTE_OPTION_NO_WITH_VETO = 4
    """VOTE_OPTION_NO_WITH_VETO defines a no with veto vote option."""


class ProposalStatus(betterproto.Enum):
    """ProposalStatus enumerates the valid statuses of a proposal."""

    PROPOSAL_STATUS_UNSPECIFIED = 0
    """PROPOSAL_STATUS_UNSPECIFIED defines the default propopsal status."""

    PROPOSAL_STATUS_DEPOSIT_PERIOD = 1
    """
    PROPOSAL_STATUS_DEPOSIT_PERIOD defines a proposal status during the deposit
    period.
    """

    PROPOSAL_STATUS_VOTING_PERIOD = 2
    """
    PROPOSAL_STATUS_VOTING_PERIOD defines a proposal status during the voting
    period.
    """

    PROPOSAL_STATUS_PASSED = 3
    """
    PROPOSAL_STATUS_PASSED defines a proposal status of a proposal that has
    passed.
    """

    PROPOSAL_STATUS_REJECTED = 4
    """
    PROPOSAL_STATUS_REJECTED defines a proposal status of a proposal that has
    been rejected.
    """

    PROPOSAL_STATUS_FAILED = 5
    """
    PROPOSAL_STATUS_FAILED defines a proposal status of a proposal that has
    failed.
    """


@dataclass(eq=False, repr=False)
class WeightedVoteOption(betterproto.Message):
    """
    WeightedVoteOption defines a unit of vote for vote split. Since: cosmos-sdk
    0.43
    """

    option: "VoteOption" = betterproto.enum_field(1)
    weight: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class TextProposal(betterproto.Message):
    """
    TextProposal defines a standard text proposal whose changes need to be
    manually updated in case of approval.
    """

    title: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class Deposit(betterproto.Message):
    """
    Deposit defines an amount deposited by an account address to an active
    proposal.
    """

    proposal_id: int = betterproto.uint64_field(1)
    depositor: str = betterproto.string_field(2)
    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class Proposal(betterproto.Message):
    """Proposal defines the core field members of a governance proposal."""

    proposal_id: int = betterproto.uint64_field(1)
    content: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)
    status: "ProposalStatus" = betterproto.enum_field(3)
    final_tally_result: "TallyResult" = betterproto.message_field(4)
    submit_time: datetime = betterproto.message_field(5)
    deposit_end_time: datetime = betterproto.message_field(6)
    total_deposit: List["__base_v1_beta1__.Coin"] = betterproto.message_field(7)
    voting_start_time: datetime = betterproto.message_field(8)
    voting_end_time: datetime = betterproto.message_field(9)


@dataclass(eq=False, repr=False)
class TallyResult(betterproto.Message):
    """TallyResult defines a standard tally for a governance proposal."""

    yes: str = betterproto.string_field(1)
    abstain: str = betterproto.string_field(2)
    no: str = betterproto.string_field(3)
    no_with_veto: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class Vote(betterproto.Message):
    """
    Vote defines a vote on a governance proposal. A Vote consists of a proposal
    ID, the voter, and the vote option.
    """

    proposal_id: int = betterproto.uint64_field(1)
    voter: str = betterproto.string_field(2)
    option: "VoteOption" = betterproto.enum_field(3)
    """
    Deprecated: Prefer to use `options` instead. This field is set in queries
    if and only if `len(options) == 1` and that option has weight 1. In all
    other cases, this field will default to VOTE_OPTION_UNSPECIFIED.
    """

    options: List["WeightedVoteOption"] = betterproto.message_field(4)
    """Since: cosmos-sdk 0.43"""

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("option"):
            warnings.warn("Vote.option is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class DepositParams(betterproto.Message):
    """
    DepositParams defines the params for deposits on governance proposals.
    """

    min_deposit: List["__base_v1_beta1__.Coin"] = betterproto.message_field(1)
    """Minimum deposit for a proposal to enter voting period."""

    max_deposit_period: timedelta = betterproto.message_field(2)
    """
    Maximum period for Atom holders to deposit on a proposal. Initial value: 2
    months.
    """


@dataclass(eq=False, repr=False)
class VotingParams(betterproto.Message):
    """VotingParams defines the params for voting on governance proposals."""

    voting_period: timedelta = betterproto.message_field(1)
    """Length of the voting period."""


@dataclass(eq=False, repr=False)
class TallyParams(betterproto.Message):
    """
    TallyParams defines the params for tallying votes on governance proposals.
    """

    quorum: bytes = betterproto.bytes_field(1)
    """
    Minimum percentage of total stake needed to vote for a result to be
    considered valid.
    """

    threshold: bytes = betterproto.bytes_field(2)
    """
    Minimum proportion of Yes votes for proposal to pass. Default value: 0.5.
    """

    veto_threshold: bytes = betterproto.bytes_field(3)
    """
    Minimum value of Veto votes to Total votes ratio for proposal to be
    vetoed. Default value: 1/3.
    """


@dataclass(eq=False, repr=False)
class MsgSubmitProposal(betterproto.Message):
    """
    MsgSubmitProposal defines an sdk.Msg type that supports submitting
    arbitrary proposal Content.
    """

    content: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    initial_deposit: List["__base_v1_beta1__.Coin"] = betterproto.message_field(2)
    proposer: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class MsgSubmitProposalResponse(betterproto.Message):
    """
    MsgSubmitProposalResponse defines the Msg/SubmitProposal response type.
    """

    proposal_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class MsgVote(betterproto.Message):
    """MsgVote defines a message to cast a vote."""

    proposal_id: int = betterproto.uint64_field(1)
    voter: str = betterproto.string_field(2)
    option: "VoteOption" = betterproto.enum_field(3)


@dataclass(eq=False, repr=False)
class MsgVoteResponse(betterproto.Message):
    """MsgVoteResponse defines the Msg/Vote response type."""

    pass


@dataclass(eq=False, repr=False)
class MsgVoteWeighted(betterproto.Message):
    """
    MsgVoteWeighted defines a message to cast a vote. Since: cosmos-sdk 0.43
    """

    proposal_id: int = betterproto.uint64_field(1)
    voter: str = betterproto.string_field(2)
    options: List["WeightedVoteOption"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MsgVoteWeightedResponse(betterproto.Message):
    """
    MsgVoteWeightedResponse defines the Msg/VoteWeighted response type. Since:
    cosmos-sdk 0.43
    """

    pass


@dataclass(eq=False, repr=False)
class MsgDeposit(betterproto.Message):
    """
    MsgDeposit defines a message to submit a deposit to an existing proposal.
    """

    proposal_id: int = betterproto.uint64_field(1)
    depositor: str = betterproto.string_field(2)
    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MsgDepositResponse(betterproto.Message):
    """MsgDepositResponse defines the Msg/Deposit response type."""

    pass


@dataclass(eq=False, repr=False)
class QueryProposalRequest(betterproto.Message):
    """
    QueryProposalRequest is the request type for the Query/Proposal RPC method.
    """

    proposal_id: int = betterproto.uint64_field(1)
    """proposal_id defines the unique id of the proposal."""


@dataclass(eq=False, repr=False)
class QueryProposalResponse(betterproto.Message):
    """
    QueryProposalResponse is the response type for the Query/Proposal RPC
    method.
    """

    proposal: "Proposal" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryProposalsRequest(betterproto.Message):
    """
    QueryProposalsRequest is the request type for the Query/Proposals RPC
    method.
    """

    proposal_status: "ProposalStatus" = betterproto.enum_field(1)
    """proposal_status defines the status of the proposals."""

    voter: str = betterproto.string_field(2)
    """voter defines the voter address for the proposals."""

    depositor: str = betterproto.string_field(3)
    """depositor defines the deposit addresses from the proposals."""

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(4)
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryProposalsResponse(betterproto.Message):
    """
    QueryProposalsResponse is the response type for the Query/Proposals RPC
    method.
    """

    proposals: List["Proposal"] = betterproto.message_field(1)
    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryVoteRequest(betterproto.Message):
    """QueryVoteRequest is the request type for the Query/Vote RPC method."""

    proposal_id: int = betterproto.uint64_field(1)
    """proposal_id defines the unique id of the proposal."""

    voter: str = betterproto.string_field(2)
    """voter defines the oter address for the proposals."""


@dataclass(eq=False, repr=False)
class QueryVoteResponse(betterproto.Message):
    """
    QueryVoteResponse is the response type for the Query/Vote RPC method.
    """

    vote: "Vote" = betterproto.message_field(1)
    """vote defined the queried vote."""


@dataclass(eq=False, repr=False)
class QueryVotesRequest(betterproto.Message):
    """
    QueryVotesRequest is the request type for the Query/Votes RPC method.
    """

    proposal_id: int = betterproto.uint64_field(1)
    """proposal_id defines the unique id of the proposal."""

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryVotesResponse(betterproto.Message):
    """
    QueryVotesResponse is the response type for the Query/Votes RPC method.
    """

    votes: List["Vote"] = betterproto.message_field(1)
    """votes defined the queried votes."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    params_type: str = betterproto.string_field(1)
    """
    params_type defines which parameters to query for, can be one of "voting",
    "tallying" or "deposit".
    """


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method.
    """

    voting_params: "VotingParams" = betterproto.message_field(1)
    """voting_params defines the parameters related to voting."""

    deposit_params: "DepositParams" = betterproto.message_field(2)
    """deposit_params defines the parameters related to deposit."""

    tally_params: "TallyParams" = betterproto.message_field(3)
    """tally_params defines the parameters related to tally."""


@dataclass(eq=False, repr=False)
class QueryDepositRequest(betterproto.Message):
    """
    QueryDepositRequest is the request type for the Query/Deposit RPC method.
    """

    proposal_id: int = betterproto.uint64_field(1)
    """proposal_id defines the unique id of the proposal."""

    depositor: str = betterproto.string_field(2)
    """depositor defines the deposit addresses from the proposals."""


@dataclass(eq=False, repr=False)
class QueryDepositResponse(betterproto.Message):
    """
    QueryDepositResponse is the response type for the Query/Deposit RPC method.
    """

    deposit: "Deposit" = betterproto.message_field(1)
    """deposit defines the requested deposit."""


@dataclass(eq=False, repr=False)
class QueryDepositsRequest(betterproto.Message):
    """
    QueryDepositsRequest is the request type for the Query/Deposits RPC method.
    """

    proposal_id: int = betterproto.uint64_field(1)
    """proposal_id defines the unique id of the proposal."""

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryDepositsResponse(betterproto.Message):
    """
    QueryDepositsResponse is the response type for the Query/Deposits RPC
    method.
    """

    deposits: List["Deposit"] = betterproto.message_field(1)
    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryTallyResultRequest(betterproto.Message):
    """
    QueryTallyResultRequest is the request type for the Query/Tally RPC method.
    """

    proposal_id: int = betterproto.uint64_field(1)
    """proposal_id defines the unique id of the proposal."""


@dataclass(eq=False, repr=False)
class QueryTallyResultResponse(betterproto.Message):
    """
    QueryTallyResultResponse is the response type for the Query/Tally RPC
    method.
    """

    tally: "TallyResult" = betterproto.message_field(1)
    """tally defines the requested tally."""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the gov module's genesis state."""

    starting_proposal_id: int = betterproto.uint64_field(1)
    """starting_proposal_id is the ID of the starting proposal."""

    deposits: List["Deposit"] = betterproto.message_field(2)
    """deposits defines all the deposits present at genesis."""

    votes: List["Vote"] = betterproto.message_field(3)
    """votes defines all the votes present at genesis."""

    proposals: List["Proposal"] = betterproto.message_field(4)
    """proposals defines all the proposals present at genesis."""

    deposit_params: "DepositParams" = betterproto.message_field(5)
    """params defines all the paramaters of related to deposit."""

    voting_params: "VotingParams" = betterproto.message_field(6)
    """params defines all the paramaters of related to voting."""

    tally_params: "TallyParams" = betterproto.message_field(7)
    """params defines all the paramaters of related to tally."""


class MsgStub(betterproto.ServiceStub):
    async def submit_proposal(
        self,
        msg_submit_proposal: "MsgSubmitProposal",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgSubmitProposalResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Msg/SubmitProposal",
            msg_submit_proposal,
            MsgSubmitProposalResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def vote(
        self,
        msg_vote: "MsgVote",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgVoteResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Msg/Vote",
            msg_vote,
            MsgVoteResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def vote_weighted(
        self,
        msg_vote_weighted: "MsgVoteWeighted",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgVoteWeightedResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Msg/VoteWeighted",
            msg_vote_weighted,
            MsgVoteWeightedResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def deposit(
        self,
        msg_deposit: "MsgDeposit",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgDepositResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Msg/Deposit",
            msg_deposit,
            MsgDepositResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def proposal(
        self,
        query_proposal_request: "QueryProposalRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryProposalResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Query/Proposal",
            query_proposal_request,
            QueryProposalResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def proposals(
        self,
        query_proposals_request: "QueryProposalsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryProposalsResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Query/Proposals",
            query_proposals_request,
            QueryProposalsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def vote(
        self,
        query_vote_request: "QueryVoteRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryVoteResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Query/Vote",
            query_vote_request,
            QueryVoteResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def votes(
        self,
        query_votes_request: "QueryVotesRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryVotesResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Query/Votes",
            query_votes_request,
            QueryVotesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def deposit(
        self,
        query_deposit_request: "QueryDepositRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryDepositResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Query/Deposit",
            query_deposit_request,
            QueryDepositResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def deposits(
        self,
        query_deposits_request: "QueryDepositsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryDepositsResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Query/Deposits",
            query_deposits_request,
            QueryDepositsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def tally_result(
        self,
        query_tally_result_request: "QueryTallyResultRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryTallyResultResponse":
        return await self._unary_unary(
            "/cosmos.gov.v1beta1.Query/TallyResult",
            query_tally_result_request,
            QueryTallyResultResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def submit_proposal(self, msg_submit_proposal: "MsgSubmitProposal") -> "MsgSubmitProposalResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def vote(self, msg_vote: "MsgVote") -> "MsgVoteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def vote_weighted(self, msg_vote_weighted: "MsgVoteWeighted") -> "MsgVoteWeightedResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def deposit(self, msg_deposit: "MsgDeposit") -> "MsgDepositResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_submit_proposal(
        self,
        stream: "grpclib.server.Stream[MsgSubmitProposal, MsgSubmitProposalResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.submit_proposal(request)
        await stream.send_message(response)

    async def __rpc_vote(self, stream: "grpclib.server.Stream[MsgVote, MsgVoteResponse]") -> None:
        request = await stream.recv_message()
        response = await self.vote(request)
        await stream.send_message(response)

    async def __rpc_vote_weighted(
        self, stream: "grpclib.server.Stream[MsgVoteWeighted, MsgVoteWeightedResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.vote_weighted(request)
        await stream.send_message(response)

    async def __rpc_deposit(self, stream: "grpclib.server.Stream[MsgDeposit, MsgDepositResponse]") -> None:
        request = await stream.recv_message()
        response = await self.deposit(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.gov.v1beta1.Msg/SubmitProposal": grpclib.const.Handler(
                self.__rpc_submit_proposal,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgSubmitProposal,
                MsgSubmitProposalResponse,
            ),
            "/cosmos.gov.v1beta1.Msg/Vote": grpclib.const.Handler(
                self.__rpc_vote,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgVote,
                MsgVoteResponse,
            ),
            "/cosmos.gov.v1beta1.Msg/VoteWeighted": grpclib.const.Handler(
                self.__rpc_vote_weighted,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgVoteWeighted,
                MsgVoteWeightedResponse,
            ),
            "/cosmos.gov.v1beta1.Msg/Deposit": grpclib.const.Handler(
                self.__rpc_deposit,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgDeposit,
                MsgDepositResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def proposal(self, query_proposal_request: "QueryProposalRequest") -> "QueryProposalResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def proposals(self, query_proposals_request: "QueryProposalsRequest") -> "QueryProposalsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def vote(self, query_vote_request: "QueryVoteRequest") -> "QueryVoteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def votes(self, query_votes_request: "QueryVotesRequest") -> "QueryVotesResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(self, query_params_request: "QueryParamsRequest") -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def deposit(self, query_deposit_request: "QueryDepositRequest") -> "QueryDepositResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def deposits(self, query_deposits_request: "QueryDepositsRequest") -> "QueryDepositsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def tally_result(self, query_tally_result_request: "QueryTallyResultRequest") -> "QueryTallyResultResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_proposal(
        self,
        stream: "grpclib.server.Stream[QueryProposalRequest, QueryProposalResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.proposal(request)
        await stream.send_message(response)

    async def __rpc_proposals(
        self,
        stream: "grpclib.server.Stream[QueryProposalsRequest, QueryProposalsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.proposals(request)
        await stream.send_message(response)

    async def __rpc_vote(self, stream: "grpclib.server.Stream[QueryVoteRequest, QueryVoteResponse]") -> None:
        request = await stream.recv_message()
        response = await self.vote(request)
        await stream.send_message(response)

    async def __rpc_votes(self, stream: "grpclib.server.Stream[QueryVotesRequest, QueryVotesResponse]") -> None:
        request = await stream.recv_message()
        response = await self.votes(request)
        await stream.send_message(response)

    async def __rpc_params(self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_deposit(self, stream: "grpclib.server.Stream[QueryDepositRequest, QueryDepositResponse]") -> None:
        request = await stream.recv_message()
        response = await self.deposit(request)
        await stream.send_message(response)

    async def __rpc_deposits(
        self,
        stream: "grpclib.server.Stream[QueryDepositsRequest, QueryDepositsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.deposits(request)
        await stream.send_message(response)

    async def __rpc_tally_result(
        self,
        stream: "grpclib.server.Stream[QueryTallyResultRequest, QueryTallyResultResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.tally_result(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.gov.v1beta1.Query/Proposal": grpclib.const.Handler(
                self.__rpc_proposal,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryProposalRequest,
                QueryProposalResponse,
            ),
            "/cosmos.gov.v1beta1.Query/Proposals": grpclib.const.Handler(
                self.__rpc_proposals,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryProposalsRequest,
                QueryProposalsResponse,
            ),
            "/cosmos.gov.v1beta1.Query/Vote": grpclib.const.Handler(
                self.__rpc_vote,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryVoteRequest,
                QueryVoteResponse,
            ),
            "/cosmos.gov.v1beta1.Query/Votes": grpclib.const.Handler(
                self.__rpc_votes,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryVotesRequest,
                QueryVotesResponse,
            ),
            "/cosmos.gov.v1beta1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            "/cosmos.gov.v1beta1.Query/Deposit": grpclib.const.Handler(
                self.__rpc_deposit,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDepositRequest,
                QueryDepositResponse,
            ),
            "/cosmos.gov.v1beta1.Query/Deposits": grpclib.const.Handler(
                self.__rpc_deposits,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDepositsRequest,
                QueryDepositsResponse,
            ),
            "/cosmos.gov.v1beta1.Query/TallyResult": grpclib.const.Handler(
                self.__rpc_tally_result,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryTallyResultRequest,
                QueryTallyResultResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: confio/proofs.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


class HashOp(betterproto.Enum):
    NO_HASH = 0
    """
    NO_HASH is the default if no data passed. Note this is an illegal argument
    some places.
    """

    SHA256 = 1
    SHA512 = 2
    KECCAK = 3
    RIPEMD160 = 4
    BITCOIN = 5


class LengthOp(betterproto.Enum):
    """
    *LengthOp defines how to process the key and value of the LeafOpto include
    length information. After encoding the length with the givenalgorithm, the
    length will be prepended to the key and value bytes.(Each one with it's own
    encoded length)
    """

    NO_PREFIX = 0
    """NO_PREFIX don't include any length info"""

    VAR_PROTO = 1
    """VAR_PROTO uses protobuf (and go-amino) varint encoding of the length"""

    VAR_RLP = 2
    """VAR_RLP uses rlp int encoding of the length"""

    FIXED32_BIG = 3
    """
    FIXED32_BIG uses big-endian encoding of the length as a 32 bit integer
    """

    FIXED32_LITTLE = 4
    """
    FIXED32_LITTLE uses little-endian encoding of the length as a 32 bit
    integer
    """

    FIXED64_BIG = 5
    """
    FIXED64_BIG uses big-endian encoding of the length as a 64 bit integer
    """

    FIXED64_LITTLE = 6
    """
    FIXED64_LITTLE uses little-endian encoding of the length as a 64 bit
    integer
    """

    REQUIRE_32_BYTES = 7
    """
    REQUIRE_32_BYTES is like NONE, but will fail if the input is not exactly 32
    bytes (sha256 output)
    """

    REQUIRE_64_BYTES = 8
    """
    REQUIRE_64_BYTES is like NONE, but will fail if the input is not exactly 64
    bytes (sha512 output)
    """


@dataclass(eq=False, repr=False)
class ExistenceProof(betterproto.Message):
    """
    *ExistenceProof takes a key and a value and a set of steps to perform on
    it.The result of peforming all these steps will provide a "root hash",
    which canbe compared to the value in a header.Since it is computationally
    infeasible to produce a hash collission for any of the usedcryptographic
    hash functions, if someone can provide a series of operations to transforma
    given key and value into a root hash that matches some trusted root, these
    key and valuesmust be in the referenced merkle tree.The only possible issue
    is maliablity in LeafOp, such as providing extra prefix data,which should
    be controlled by a spec. Eg. with lengthOp as NONE,prefix = FOO, key = BAR,
    value = CHOICEandprefix = F, key = OOBAR, value = CHOICEwould produce the
    same value.With LengthOp this is tricker but not impossible. Which is why
    the "leafPrefixEqual" fieldin the ProofSpec is valuable to prevent this
    mutability. And why all trees shouldlength-prefix the data before hashing
    it.
    """

    key: bytes = betterproto.bytes_field(1)
    value: bytes = betterproto.bytes_field(2)
    leaf: "LeafOp" = betterproto.message_field(3)
    path: List["InnerOp"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class NonExistenceProof(betterproto.Message):
    """
    NonExistenceProof takes a proof of two neighbors, one left of the desired
    key,one right of the desired key. If both proofs are valid AND they are
    neighbors,then there is no valid proof for the given key.
    """

    key: bytes = betterproto.bytes_field(1)
    left: "ExistenceProof" = betterproto.message_field(2)
    right: "ExistenceProof" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CommitmentProof(betterproto.Message):
    """
    CommitmentProof is either an ExistenceProof or a NonExistenceProof, or a
    Batch of such messages
    """

    exist: "ExistenceProof" = betterproto.message_field(1, group="proof")
    nonexist: "NonExistenceProof" = betterproto.message_field(2, group="proof")
    batch: "BatchProof" = betterproto.message_field(3, group="proof")
    compressed: "CompressedBatchProof" = betterproto.message_field(4, group="proof")


@dataclass(eq=False, repr=False)
class LeafOp(betterproto.Message):
    """
    *LeafOp represents the raw key-value data we wish to prove, andmust be
    flexible to represent the internal transformation fromthe original key-
    value pairs into the basis hash, for many existingmerkle trees.key and
    value are passed in. So that the signature of this operation is:leafOp(key,
    value) -> outputTo process this, first prehash the keys and values if
    needed (ANY means no hash in this case):hkey = prehashKey(key)hvalue =
    prehashValue(value)Then combine the bytes, and hash itoutput = hash(prefix
    || length(hkey) || hkey || length(hvalue) || hvalue)
    """

    hash: "HashOp" = betterproto.enum_field(1)
    prehash_key: "HashOp" = betterproto.enum_field(2)
    prehash_value: "HashOp" = betterproto.enum_field(3)
    length: "LengthOp" = betterproto.enum_field(4)
    prefix: bytes = betterproto.bytes_field(5)
    """
    prefix is a fixed bytes that may optionally be included at the beginning to
    differentiate a leaf node from an inner node.
    """


@dataclass(eq=False, repr=False)
class InnerOp(betterproto.Message):
    """
    *InnerOp represents a merkle-proof step that is not a leaf.It represents
    concatenating two children and hashing them to provide the next result.The
    result of the previous step is passed in, so the signature of this op
    is:innerOp(child) -> outputThe result of applying InnerOp should be:output
    = op.hash(op.prefix || child || op.suffix)where the || operator is
    concatenation of binary data,and child is the result of hashing all the
    tree below this step.Any special data, like prepending child with the
    length, or prepending the entire operation withsome value to differentiate
    from leaf nodes, should be included in prefix and suffix.If either of
    prefix or suffix is empty, we just treat it as an empty string
    """

    hash: "HashOp" = betterproto.enum_field(1)
    prefix: bytes = betterproto.bytes_field(2)
    suffix: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class ProofSpec(betterproto.Message):
    """
    *ProofSpec defines what the expected parameters are for a given proof
    type.This can be stored in the client and used to validate any incoming
    proofs.verify(ProofSpec, Proof) -> Proof | ErrorAs demonstrated in tests,
    if we don't fix the algorithm used to calculate theLeafHash for a given
    tree, there are many possible key-value pairs that cangenerate a given hash
    (by interpretting the preimage differently).We need this for proper
    security, requires client knows a priori whattree format server uses. But
    not in code, rather a configuration object.
    """

    leaf_spec: "LeafOp" = betterproto.message_field(1)
    """
    any field in the ExistenceProof must be the same as in this spec. except
    Prefix, which is just the first bytes of prefix (spec can be longer)
    """

    inner_spec: "InnerSpec" = betterproto.message_field(2)
    max_depth: int = betterproto.int32_field(3)
    """
    max_depth (if > 0) is the maximum number of InnerOps allowed (mainly for
    fixed-depth tries)
    """

    min_depth: int = betterproto.int32_field(4)
    """
    min_depth (if > 0) is the minimum number of InnerOps allowed (mainly for
    fixed-depth tries)
    """


@dataclass(eq=False, repr=False)
class InnerSpec(betterproto.Message):
    """
    InnerSpec contains all store-specific structure info to determine if two
    proofs from agiven store are neighbors.This enables:isLeftMost(spec:
    InnerSpec, op: InnerOp)isRightMost(spec: InnerSpec, op:
    InnerOp)isLeftNeighbor(spec: InnerSpec, left: InnerOp, right: InnerOp)
    """

    child_order: List[int] = betterproto.int32_field(1)
    """
    Child order is the ordering of the children node, must count from 0 iavl
    tree is [0, 1] (left then right) merk is [0, 2, 1] (left, right, here)
    """

    child_size: int = betterproto.int32_field(2)
    min_prefix_length: int = betterproto.int32_field(3)
    max_prefix_length: int = betterproto.int32_field(4)
    empty_child: bytes = betterproto.bytes_field(5)
    """
    empty child is the prehash image that is used when one child is nil (eg. 20
    bytes of 0)
    """

    hash: "HashOp" = betterproto.enum_field(6)
    """hash is the algorithm that must be used for each InnerOp"""


@dataclass(eq=False, repr=False)
class BatchProof(betterproto.Message):
    """BatchProof is a group of multiple proof types than can be compressed"""

    entries: List["BatchEntry"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class BatchEntry(betterproto.Message):
    """Use BatchEntry not CommitmentProof, to avoid recursion"""

    exist: "ExistenceProof" = betterproto.message_field(1, group="proof")
    nonexist: "NonExistenceProof" = betterproto.message_field(2, group="proof")


@dataclass(eq=False, repr=False)
class CompressedBatchProof(betterproto.Message):
    entries: List["CompressedBatchEntry"] = betterproto.message_field(1)
    lookup_inners: List["InnerOp"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CompressedBatchEntry(betterproto.Message):
    """Use BatchEntry not CommitmentProof, to avoid recursion"""

    exist: "CompressedExistenceProof" = betterproto.message_field(1, group="proof")
    nonexist: "CompressedNonExistenceProof" = betterproto.message_field(2, group="proof")


@dataclass(eq=False, repr=False)
class CompressedExistenceProof(betterproto.Message):
    key: bytes = betterproto.bytes_field(1)
    value: bytes = betterproto.bytes_field(2)
    leaf: "LeafOp" = betterproto.message_field(3)
    path: List[int] = betterproto.int32_field(4)
    """
    these are indexes into the lookup_inners table in CompressedBatchProof
    """


@dataclass(eq=False, repr=False)
class CompressedNonExistenceProof(betterproto.Message):
    key: bytes = betterproto.bytes_field(1)
    left: "CompressedExistenceProof" = betterproto.message_field(2)
    right: "CompressedExistenceProof" = betterproto.message_field(3)
